@@ -15,7 +15,6 @@
 #include <QTemporaryFile>
 #include <QTest>
 #include <QVector>
-#include <QDebug>
 
 // note: this test request several data from a real example journald database
 //       you can check them by using "journalctl -D journal" and requesting the values
@@ -40,6 +39,7 @@ void TestViewModel::rowAccess()
 {
     // used unfiltered journal and check first two lines
     JournaldViewModel model;
+
     QAbstractItemModelTester tester(&model, QAbstractItemModelTester::FailureReportingMode::QtTest);
     QCOMPARE(model.setJournaldPath(JOURNAL_LOCATION), true);
     QVERIFY(model.rowCount() > 0);
@@ -77,23 +77,27 @@ void TestViewModel::rowAccess()
 void TestViewModel::bootFilter()
 {
     JournaldViewModel model;
+    Filter filter = model.filter();
     QAbstractItemModelTester tester(&model, QAbstractItemModelTester::FailureReportingMode::QtTest);
     QCOMPARE(model.setJournaldPath(JOURNAL_LOCATION), true);
 
     // select only second boot
-    model.setBootFilter({mBoots.at(1)});
+    filter.setBootFilter({mBoots.at(1)});
+    model.setFilter(filter);
     QVERIFY(model.rowCount() > 0);
     QCOMPARE(model.data(model.index(0, 0), JournaldViewModel::BOOT_ID), mBoots.at(1));
 
     // select only first boot
-    model.setBootFilter({mBoots.at(0)});
+    filter.setBootFilter({mBoots.at(0)});
+    model.setFilter(filter);
     QVERIFY(model.rowCount() > 0);
     QCOMPARE(model.data(model.index(0, 0), JournaldViewModel::BOOT_ID), mBoots.at(0));
 
     // select first and second boot
     bool firstBootFound = false;
     bool secondBootFound = false;
-    model.setBootFilter({mBoots.at(0), mBoots.at(1)});
+    filter.setBootFilter({mBoots.at(0), mBoots.at(1)});
+    model.setFilter(filter);
     // value of 20.000 is chosen arbitrarily from test with this journal archive
     while (model.canFetchMore(QModelIndex()) && model.rowCount() < 20'000) {
         model.fetchMore(QModelIndex());
@@ -112,18 +116,22 @@ void TestViewModel::bootFilter()
 void TestViewModel::unitFilter()
 {
     JournaldViewModel model;
+    Filter filter = model.filter();
     QAbstractItemModelTester tester(&model, QAbstractItemModelTester::FailureReportingMode::QtTest);
     QCOMPARE(model.setJournaldPath(JOURNAL_LOCATION), true);
 
     // select single service
-    model.setSystemdUnitFilter({"systemd-networkd.service"});
+
+    filter.setSystemdUnitFilter({"systemd-networkd.service"});
+    model.setFilter(filter);
     QVERIFY(model.rowCount() > 0);
     QCOMPARE(model.data(model.index(0, 0), JournaldViewModel::SYSTEMD_UNIT), "systemd-networkd.service");
 
     // test mulitple services
     QStringList testSystemdUnitNames{"init.scope", "dbus.service", "systemd-networkd.service"};
     QStringList notFoundUnits = testSystemdUnitNames;
-    model.setSystemdUnitFilter(testSystemdUnitNames);
+    filter.setSystemdUnitFilter(testSystemdUnitNames);
+    model.setFilter(filter);
 
     while (model.canFetchMore(QModelIndex())) {
         model.fetchMore(QModelIndex());
@@ -143,34 +151,41 @@ void TestViewModel::unitFilter()
 void TestViewModel::showKernelMessages()
 {
     // obtained string with:
-    // journalctl -b -0 -k -D . -o cat | head -n1
-    const QString arbitraryKernelMessage = "brcmfmac: brcmf_fw_alloc_request: using brcm/brcmfmac43455-sdio for chip BCM4345/6";
+    // journalctl _TRANSPORT=kernel -b 2dbe99dd855049af8f2865c5da2b8fda -D . -o json|head -1
+    const QString needleCursor = "s=71e0819dabc84edab6c4a3b6f386e57a;i=195;b=2dbe99dd855049af8f2865c5da2b8fda;m=3f55c8;t=5bd6cd5f8b895;x=c3f4327cdb1c3b6f";
+    const QString needleMessage = "brcmfmac: brcmf_fw_alloc_request: using brcm/brcmfmac43455-sdio for chip BCM4345/6";
 
     JournaldViewModel model;
+    Filter filter = model.filter();
+    filter.setBootFilter({"2dbe99dd855049af8f2865c5da2b8fda"});
     QAbstractItemModelTester tester(&model, QAbstractItemModelTester::FailureReportingMode::QtTest);
     QCOMPARE(model.setJournaldPath(JOURNAL_LOCATION), true);
 
-    // check that not contains Kernel message
-    model.setKernelFilter(false);
+    // check that journal does not contain Kernel message
+    filter.setKernelMessagesEnabled(false);
+    model.setFilter(filter);
     QVERIFY(model.rowCount() > 0);
-    while (model.canFetchMore(QModelIndex())  && model.rowCount() < 2000) {
+    while (model.canFetchMore(QModelIndex()) && model.rowCount() < 2000) {
         model.fetchMore(QModelIndex());
     }
     for (int i = 0; i < model.rowCount(); ++i) {
         const QString message = model.data(model.index(i, 0), JournaldViewModel::MESSAGE).toString();
-        QVERIFY(arbitraryKernelMessage != message);
+        QVERIFY(needleMessage != message);
     }
 
     // check that Kernel messages are containted
-    model.setKernelFilter(true);
+    filter.setKernelMessagesEnabled(true);
+    model.setFilter(filter);
     QVERIFY(model.rowCount() > 0);
-    while (model.canFetchMore(QModelIndex())) {
+    // 10 is magic number to avoid full model fetching
+    for (int i = 0; i < 10 && model.canFetchMore(QModelIndex()); ++i) {
         model.fetchMore(QModelIndex());
     }
     bool found{false};
     for (int i = 0; i < model.rowCount(); ++i) {
-        const QString message = model.data(model.index(i, 0), JournaldViewModel::MESSAGE).toString();
-        if (arbitraryKernelMessage == message) {
+        const QString cursor = model.data(model.index(i, 0), JournaldViewModel::CURSOR).toString();
+        if (needleCursor == cursor) {
+            QCOMPARE(model.data(model.index(i, 0), JournaldViewModel::MESSAGE).toString(), needleMessage);
             found = true;
         }
     }
@@ -205,8 +220,10 @@ void TestViewModel::readFullJournal()
     JournaldViewModel model;
     QAbstractItemModelTester tester(&model, QAbstractItemModelTester::FailureReportingMode::QtTest);
     QCOMPARE(model.setJournaldPath(JOURNAL_LOCATION), true);
-    model.setBootFilter({mBoots.at(0)});
-    model.setKernelFilter(true);
+    Filter filter = model.filter();
+    filter.setBootFilter({mBoots.at(0)});
+    filter.setKernelMessagesEnabled(true);
+    model.setFilter(filter);
 
     qDebug().noquote() << "check with:"
                        << "journalctl -b" << mBoots.at(0);
@@ -225,6 +242,7 @@ void TestViewModel::readFullJournal()
 void TestViewModel::resetModelHeadAndTailCursorTest()
 {
     JournaldViewModel model;
+    Filter filter = model.filter();
     QAbstractItemModelTester tester(&model, QAbstractItemModelTester::FailureReportingMode::QtTest);
     QCOMPARE(model.setJournaldPath(JOURNAL_LOCATION), true);
 
@@ -240,7 +258,8 @@ void TestViewModel::resetModelHeadAndTailCursorTest()
          "s=f1a33fced0cb4d2bb629fb2bd70d1326;i=48e;b=27acae2fe35a40ac93f9c7732c0b8e59;m=67f0d79;t=5bd6cd098ce95;x=c20fdd57f02ca4c5"}};
 
     // use model and set boot 0
-    model.setBootFilter({mBoots.at(0)});
+    filter.setBootFilter({mBoots.at(0)});
+    model.setFilter(filter);
     // boot has 925 non-kernel log entries, fetching once with chunk size 500 gets all
     model.fetchMore(QModelIndex());
 
@@ -262,7 +281,8 @@ void TestViewModel::resetModelHeadAndTailCursorTest()
     QCOMPARE(model.data(model.index(model.rowCount() - 1, 0), JournaldViewModel::CURSOR), cursors.at(0).mTail);
 
     // use model and set boot 1
-    model.setBootFilter({mBoots.at(1)});
+    filter.setBootFilter({mBoots.at(1)});
+    model.setFilter(filter);
     model.fetchMore(QModelIndex());
     // journalctl -b 27acae2fe35a40ac93f9c7732c0b8e59 -D . -o json|head -n1
     QCOMPARE(model.data(model.index(0, 0), JournaldViewModel::CURSOR), cursors.at(1).mHead);
@@ -309,7 +329,9 @@ void TestViewModel::stringSearch()
     // simple check when full journal is alread read
     {
         JournaldViewModel model;
-        model.setBootFilter({mBoots.at(0)}); // select boot -2 == 68f2e61d061247d8a8ba0b8d53a97a52
+        Filter filter = model.filter();
+        filter.setBootFilter({mBoots.at(0)}); // select boot -2 == 68f2e61d061247d8a8ba0b8d53a97a52
+        model.setFilter(filter);
         QAbstractItemModelTester tester(&model, QAbstractItemModelTester::FailureReportingMode::QtTest);
         QCOMPARE(model.setJournaldPath(JOURNAL_LOCATION), true);
         int foundLine{-1};
@@ -319,16 +341,18 @@ void TestViewModel::stringSearch()
             if (foundLine != -1) {
                 results.push_back(foundLine);
             }
-
         } while (foundLine != -1);
+        QVERIFY(results.size() > 0);
         CONTAINER_EQUAL(needleLines, results);
     }
 
     // enforce multiple chunk reads
     {
         JournaldViewModel model;
+        Filter filter = model.filter();
         model.setFetchMoreChunkSize(10);
-        model.setBootFilter({mBoots.at(0)}); // select boot -2 == 68f2e61d061247d8a8ba0b8d53a97a52
+        filter.setBootFilter({mBoots.at(0)}); // select boot -2 == 68f2e61d061247d8a8ba0b8d53a97a52
+        model.setFilter(filter);
         QAbstractItemModelTester tester(&model, QAbstractItemModelTester::FailureReportingMode::QtTest);
         QCOMPARE(model.setJournaldPath(JOURNAL_LOCATION), true);
         int foundLine{-1};
@@ -352,7 +376,9 @@ void TestViewModel::stringSearch()
     // test backward search
     {
         JournaldViewModel model;
-        model.setBootFilter({mBoots.at(0)}); // select boot -2 == 68f2e61d061247d8a8ba0b8d53a97a52
+        Filter filter = model.filter();
+        filter.setBootFilter({mBoots.at(0)}); // select boot -2 == 68f2e61d061247d8a8ba0b8d53a97a52
+        model.setFilter(filter);
         model.seekTail();
         QAbstractItemModelTester tester(&model, QAbstractItemModelTester::FailureReportingMode::QtTest);
         QCOMPARE(model.setJournaldPath(JOURNAL_LOCATION), true);
